#! /bin/sh
# the next line restarts using wish \
exec wish "$0" "$@"
#
# graph parameters
#
set scaleparam   0.7
set scaleresol   0.05
#
set d_noderad    10
set d_edgelen    100
set d_margin     40
set d_linewidth  2
set d_fontsize   14
set d_fontfamily helvetica
#
set fbfontname {times 16 bold}
set initnodecolor ivory
set initnodelabel ""
set initedgecolor black
set speed     10
#
# begin: subroutines 
#
proc fwd_draw {} {
    global pause
    global fward
    global fwardnum
    global stepcnt
    global stepmax
    global speed
    global crrval
    global bward
    #
    #puts $stepcnt
    while {[expr !0]} {
	if {$stepcnt >= $stepmax} {
	    set pause 1
	    reconfig_pausebutton
	    break
	}
	#puts $fwardnum($stepcnt)
	for {set i 0} {$i < $fwardnum($stepcnt)} {incr i} {
	    set operation [lindex $fward($stepcnt,$i) 0]
	    #puts $operation
	    switch $operation {
		label {
		    set nodeindex [lindex $fward($stepcnt,$i) 1]
		    set labeltext [lindex $fward($stepcnt,$i) 2]
		    set bward($stepcnt,$i) \
			    "label $nodeindex $crrval(nlabel$nodeindex)"
		    set crrval(nlabel$nodeindex) $labeltext
		    .bc.cans.can itemconfigure "nlabel$nodeindex" \
			    -text "$labeltext"
		}
		colornode {
		    set nodeindex [lindex $fward($stepcnt,$i) 1]
		    set color [lindex $fward($stepcnt,$i) 2]
		    set bward($stepcnt,$i) \
			    "colornode $nodeindex $crrval(node$nodeindex)"
		    set crrval(node$nodeindex) $color
		    .bc.cans.can itemconfigure "node$nodeindex" \
			    -fill "$color"
		}
		coloredge {
		    set end0 [lindex $fward($stepcnt,$i) 1]
		    set end1 [lindex $fward($stepcnt,$i) 2]
		    set color [lindex $fward($stepcnt,$i) 3]
		    #puts $color
		    if { $end0 > $end1 } {
			set temp $end0
			set end0 $end1
			set end1 $temp
		    }
		    set bward($stepcnt,$i) \
			    "coloredge $end0 $end1 $crrval(edge$end0-$end1)"
		    set crrval(edge$end0-$end1) $color
		    .bc.cans.can itemconfigure "edge$end0-$end1" \
			    -fill "$color"
		}
	    }
	}
	#set time0 [clock seconds]
	#puts $time0
	if {$pause == 0} {
	    #while { [clock seconds] < [expr $time0 + $speed] } {}
	    after [expr $speed * 10]
	}
	update
	incr stepcnt
	if {$pause == 1} { break }
    }
}
#
proc bwd_draw {} {
    global pause
    global fward
    global fwardnum
    global stepcnt
    global stepmax
    global speed
    global crrval
    global bward
    #
    #puts $stepcnt
    while {[expr !0]} {
	incr stepcnt -1
	if {$stepcnt < 0} {
	    set stepcnt 0
	    set pause 1
	    reconfig_pausebutton
	    break
	}
	for {set i 0} {$i < $fwardnum($stepcnt)} {incr i} {
	    set operation [lindex $bward($stepcnt,$i) 0]
	    switch $operation {
		label {
		    set nodeindex [lindex $bward($stepcnt,$i) 1]
		    set labeltext [lindex $bward($stepcnt,$i) 2]
		    set crrval(nlabel$nodeindex) $labeltext
		    .bc.cans.can itemconfigure "nlabel$nodeindex" \
			    -text "$labeltext"
		}
		colornode {
		    set nodeindex [lindex $bward($stepcnt,$i) 1]
		    set color [lindex $bward($stepcnt,$i) 2]
		    set crrval(node$nodeindex) $color
		    .bc.cans.can itemconfigure "node$nodeindex" \
			    -fill "$color"
		}
		coloredge {
		    set end0 [lindex $bward($stepcnt,$i) 1]
		    set end1 [lindex $bward($stepcnt,$i) 2]
		    set color [lindex $bward($stepcnt,$i) 3]
		    #puts $color
		    if { $end0 > $end1 } {
			set temp $end0
			set end0 $end1
			set end1 $temp
		    }
		    set crrval(edge$end0-$end1) $color
		    .bc.cans.can itemconfigure "edge$end0-$end1" \
			    -fill "$color"
		}
	    }
	}
	#set time0 [clock seconds]
	#puts $time0
	if {$pause == 0} {
	    #while { [clock seconds] < [expr $time0 + $speed] } {}
	    after [expr $speed * 10]
	}
	update
	if {$pause == 1} { break }
    }
}
#
proc pause_continue {} {
    global pause
    global fwdmode
    #
    set pause [expr 1 - $pause]
    reconfig_pausebutton
    if {$pause == 0} {if {$fwdmode == 1} {fwd_draw} else {bwd_draw}}
}
#
proc reconfig_pausebutton {} {
    global pause
    global fwdmode
    global stepcnt
    global stepmax
    #
    if {$stepcnt <= 0} {set fwdmode 1}
    if {$stepcnt >= $stepmax} {set fwdmode 0}
    switch $pause {
	0 { set pctext "pause" }
	1 { if {$fwdmode == 1} {set pctext "continue >>"} \
		else {set pctext "<< continue"}}
    }
    .bc.buttons.pause configure -text $pctext
}
#
proc draw_graph {} {
    global width
    global nodenum
    global edgenum
    global source
    global edge
    global noderad
    global edgelen
    global margin
    global linewidth
    global nodestart
    global fontname
    global initnodecolor
    global initnodelabel
    global initedgecolor
    global crrval
    #
    # draw edges
    for {set i 0} {$i < $edgenum} {incr i} {
	set x1 [lindex [nodec $edge($i,end0)] 0]
	set y1 [lindex [nodec $edge($i,end0)] 1]
	set x2 [lindex [nodec $edge($i,end1)] 0]
	set y2 [lindex [nodec $edge($i,end1)] 1]
	set end0 $edge($i,end0)
	set end1 $edge($i,end1)
	if { $end0 > $end1 } {
	    set temp $end0
	    set end0 $end1
	    set end1 $temp
	}
	#puts [format "edge%d (%d,%d)" $i $edge($i,end0) $edge($i,end1)]
	#puts [format " coordinates: (%g,%g)-(%g,%g)" $x1 $y1 $x2 $y2]
	.bc.cans.can create line $x1 $y1 $x2 $y2 -width $linewidth \
		-fill $initedgecolor -tags "edge$end0-$end1"
	set crrval(edge$end0-$end1) $initedgecolor
    }
    #
    # put edge labels
    for {set i 0} {$i < $edgenum} {incr i} {
	set x1 [lindex [nodec $edge($i,end0)] 0]
	set y1 [lindex [nodec $edge($i,end0)] 1]
	set x2 [lindex [nodec $edge($i,end1)] 0]
	set y2 [lindex [nodec $edge($i,end1)] 1]
	if {$x1 > $x2 || ($x1 == $x2 && $y1 < $y2)} {
	    set temp $x1
	    set x1   $x2
	    set x2   $temp
	    set temp $y1
	    set y1   $y2
	    set y2   $temp
	}
	set x3 [expr ($x1 + 2 * $x2) / 3.0]
	set y3 [expr ($y1 + 2 * $y2) / 3.0]
	if {$x1 == $x2} {
	    set anchorval w
	    set x3 [expr $x3 + 2]
	} elseif {$y1 == $y2} {
	    set anchorval s
	} elseif {$y1 < $y2} {
	    set anchorval sw
	} else {
	    set anchorval se
	}
	.bc.cans.can create text $x3 $y3 -anchor $anchorval \
		-text "$edge($i,weight)" -font fontname
    }
    #
    # draw nodes
    for {set i $nodestart} {$i < [expr $nodenum + $nodestart]} {incr i} {
	set x1 [expr [lindex [nodec $i] 0] - $noderad]
	set y1 [expr [lindex [nodec $i] 1] - $noderad]
	set x2 [expr [lindex [nodec $i] 0] + $noderad]
	set y2 [expr [lindex [nodec $i] 1] + $noderad]
	.bc.cans.can create oval $x1 $y1 $x2 $y2 -width $linewidth \
		-fill $initnodecolor -tags "node$i"
	set crrval(node$i) $initnodecolor
	if {$i == $source} {
	    # color source with different color
	    .bc.cans.can itemconfigure "node$i" -outline blue \
		    -width [expr $linewidth + 1]
	}
    }
    #
    # put node indices
    for {set i $nodestart} {$i < [expr $nodenum + $nodestart]} {incr i} {
	set x1 [lindex [nodec $i] 0]
	set y1 [lindex [nodec $i] 1]
	.bc.cans.can create text $x1 $y1 -anchor c -text "$i" -font fontname
    }
    #
    # put node labels
    for {set i $nodestart} {$i < [expr $nodenum + $nodestart]} {incr i} {
	set x1 [expr [lindex [nodec $i] 0] + $noderad]
	set y1 [expr [lindex [nodec $i] 1] - $noderad]
	.bc.cans.can create text $x1 $y1 -anchor sw -text $initnodelabel \
		-tags "nlabel$i" -font fontname -fill blue
	set crrval(nlabel$i) $initnodelabel
    }
}
#
proc nodec { nodeindex } {
    global width
    global nodenum
    global edgenum
    global source
    global edge
    global noderad
    global edgelen
    global margin
    global nodestart
    #
    set hh [expr floor( ($nodeindex - $nodestart) / $width )]
    set ww [expr $nodeindex - $nodestart - ($hh * $width)]
    set nodecx [expr $margin + $noderad + $ww * $edgelen]
    set nodecy [expr $margin + $noderad + $hh * $edgelen]
    #
    return "$nodecx $nodecy"
}
#
proc initialize_scales {} {
    global scaleparam
    global scaleresol
    global noderad
    global edgelen
    global margin
    global linewidth
    global fontsize
    global d_noderad
    global d_edgelen
    global d_margin
    global d_linewidth
    global d_fontsize
    global d_fontfamily
    #
    if {$scaleparam < $scaleresol} {set scaleparam $scaleresol}
    set noderad    [expr floor($d_noderad * $scaleparam)]
    set edgelen    [expr floor($d_edgelen * $scaleparam)]
    set margin     [expr floor($d_margin * $scaleparam)]
    set linewidth  [expr ceil($d_linewidth * $scaleparam)]
    set fontsize   [expr int(ceil($d_fontsize * $scaleparam))]
    font delete fontname
    font create fontname -family $d_fontfamily -size $fontsize -weight bold
}
#
proc initialize_parameters {} {
    global noderad
    global edgelen
    global margin
    global height
    global width
    global nodenum
    global edgenum
    global canvasw
    global canvash
    global pause
    global stepcnt
    global fwdmode
    #
    set height [expr floor( 1 + ($nodenum - 1) / $width)]
    set canvasw [expr $edgelen * ($width  - 1) + 2 * $noderad + 2 * $margin]
    set canvash [expr $edgelen * ($height - 1) + 2 * $noderad + 2 * $margin]
    set pause 1
    set stepcnt 0
    set fwdmode 1
}
#
proc read_data { datafile } {
    global width
    global nodenum
    global edgenum
    global source
    global nodestart
    global edge
    global stepmax
    global fward
    global fwardnum
    #
    set nodestart 1
    set source -1
    set stepcounter -1
    set subcounter 0
    while {[expr !0]} {
	if { [eof $datafile] } { break }
	gets $datafile str0
	if { [string first "#" $str0] == 0 } { continue }
	if { [string first "#" $str0] >= 1 } {
	    set firstapp [string first "#" $str0]
	    set str0 [ string range $str0 0 [expr $firstapp - 1] ]
	}
	set oprname [lindex $str0 0]
	switch $oprname {
	    begingraph {
		set graphmode 1
		set edgecounter 0
	    }
	    endgraph   { set graphmode 0 }
	}
	if {$graphmode == 1} {
	    switch $oprname {
		width { set width [lindex $str0 1] }
		nodenum { set nodenum [lindex $str0 1] }
		edgenum { set edgenum [lindex $str0 1] }
		source { set source [lindex $str0 1] }
		nodestart { set nodestart [lindex $str0 1] }
		edge {
		    set edge($edgecounter,weight) [lindex $str0 1]
		    set edge($edgecounter,end0) [lindex $str0 2]
		    set edge($edgecounter,end1) [lindex $str0 3]
		    set edgecounter [expr $edgecounter + 1]
		}
	    }
	} else {
	    switch $oprname {
		step {
		    if {$stepcounter >= 0} {
			set fwardnum($stepcounter) $subcounter
		    }
		    incr stepcounter
		    set subcounter 0
		}
		end {
		    if {$stepcounter >= 0} {
			set fwardnum($stepcounter) $subcounter
		    }
		    set stepmax [expr $stepcounter + 1]
		}
		default {
		    set fward($stepcounter,$subcounter) $str0
		    incr subcounter
		    #puts $subcounter
		}
	    }
	}
    }
}
#
#
proc SavePostScript {} {
    set ftypes {
        {"EPS Files" .eps}
        {"PostScript" .ps}
        {"All Files"    *}
    }
    set fname [tk_getSaveFile -filetypes $ftypes \
            -parent . -initialfile canvas -defaultextension .eps]
    if {$fname == ""} {
        return
    } else {
        .bc.cans.can postscript -file $fname -colormode color
    }
}
#
#
proc OpenDataFile {} {
    global canvasw
    global canvash
    #
    set ftypes {
        {"Data Files" .dat}
        {"All Files"    *}
    }
    set fname [tk_getOpenFile -filetypes $ftypes \
            -parent . -initialfile graphdata.dat]
    if {$fname == ""} {
        return
    } else {
        set datafile [open $fname r]
    }
    read_data $datafile
    close $datafile
    initialize_parameters
    refresh_all
}
#
proc refresh_all {} {
    global canvasw
    global canvash
    global pause
    #
    #. configure -height 500 -width 500
    .bc.cans.can configure -scrollregion "0 0 $canvasw $canvash"
    if {$canvasw > 50} { .bc.cans.can configure -width $canvasw}
    if {$canvash > 100} {
	.bc.cans.can configure -height $canvash
	.speed.sca configure -length [expr $canvash - 50]
	.scales.sca configure -length [expr $canvash - 100]
    }
    .bc.cans.can delete all
    set pause 1
    reconfig_pausebutton
    draw_graph
}
#
proc resize_all {} {
    initialize_scales
    initialize_parameters
    refresh_all
}
#
#
# end: subroutines 
#

#
# begin main routine
#

#
# initial settings for the root window
#
. configure \
	-bg white \
	-cursor hand2
#
wm title . "Draw Graph"
#
font create fontname -family $d_fontfamily
initialize_scales
#
if { $argc == 1 } {
    set datafile [open "$argv" r]
    read_data $datafile
    close $datafile
    initialize_parameters
}
#
set speedrelief raised
frame .speed
pack .speed -side left -fill both -expand yes
button .speed.labelL -text "fast" -relief $speedrelief \
	-command {incr speed -1}
button .speed.labelR -text "slow" -relief $speedrelief \
	-command {incr speed}
scale .speed.sca -from 0 -to 100 -length 450 -tickinterval 10 \
	-resolution 1 -sliderlength 20 -width 10 -showvalue yes \
	-orient vertical -variable speed -relief $speedrelief
pack .speed.labelL .speed.sca .speed.labelR -side top -fill both -expand yes
#
set scalerelief raised
frame .scales
pack .scales -side left -fill both -expand yes
button .scales.rescale -text "resize" -relief $scalerelief \
	-command resize_all
button .scales.small -text "small" -relief $scalerelief \
	-command {set scaleparam [expr $scaleparam - $scaleresol]}
button .scales.large -text "large" -relief $scalerelief \
	-command {set scaleparam [expr $scaleparam + $scaleresol]}
scale .scales.sca -from 0 -to 2 -length 400 -tickinterval 0.2 \
	-resolution $scaleresol -sliderlength 20 -width 10 -showvalue yes \
	-orient vertical -variable scaleparam -relief $scalerelief
pack .scales.rescale .scales.small .scales.sca .scales.large \
	-side top -fill both -expand yes
#
frame .bc
pack .bc -side left -fill both -expand yes
#
frame .bc.buttons
button .bc.buttons.openDAT -text "open file" -command OpenDataFile
button .bc.buttons.savePS -text "save as PS" -command SavePostScript
button .bc.buttons.fwd -text ">>" -font $fbfontname -command { 
    if {$pause == 1} {
	fwd_draw
    } else {
	set pause 1
    }
    set fwdmode 1
    reconfig_pausebutton
}
button .bc.buttons.bwd -text "<<" -font $fbfontname -command {
    if {$pause == 1} {
	bwd_draw
    } else {
	set pause 1
    }
    set fwdmode 0
    reconfig_pausebutton
}
button .bc.buttons.pause -text "continue >>" -command pause_continue
button .bc.buttons.exit -text "exit" -command { exit }
pack .bc.buttons -side top -fill both -expand yes
pack .bc.buttons.openDAT .bc.buttons.savePS .bc.buttons.pause \
	.bc.buttons.bwd .bc.buttons.fwd .bc.buttons.exit \
	-side left -fill both -expand yes
#
frame .bc.cans
pack .bc.cans -side bottom -fill both -expand yes
canvas .bc.cans.can -background ivory \
	-width 500 -height 500 \
	-scrollregion "0 0 500 500" \
	-xscrollcommand ".bc.cans.xscrl set" \
	-yscrollcommand ".bc.cans.yscrl set"
scrollbar .bc.cans.xscrl -command ".bc.cans.can xview" -orient horizontal
scrollbar .bc.cans.yscrl -command ".bc.cans.can yview"
pack .bc.cans.xscrl -side bottom -fill x
pack .bc.cans.yscrl -side right -fill y
pack .bc.cans.can -fill both -expand yes
#
if { $argc == 1 } {
    refresh_all
#    .bc.cans.can configure -scrollregion "0 0 $canvasw $canvash"
#    if {$canvasw > 500} { .bc.cans.can configure -width $canvasw}
#    if {$canvash > 500} {
#	.bc.cans.can configure -height $canvash
#	.speed.sca configure -length [expr $canvash - 50]
#    }
#    draw_graph
}
#
bind all q {exit}
